import { Component, OnInit } from '@angular/core';
import { AbstractControl, FormBuilder, FormGroup, ValidationErrors, ValidatorFn, Validators } from '@angular/forms';
import { UserService } from '../../../services/user.service';
import { Router } from '@angular/router';
import Swal from 'sweetalert2';

@Component({
    selector: 'app-sign',
    templateUrl: './sign.component.html',
    styleUrl: './sign.component.css',
})
export class SignComponent implements OnInit {
    constructor(private fb: FormBuilder, private userService: UserService, private router: Router) {}
    formGroup: FormGroup = new FormGroup({});
    login: boolean = true;
    signup: boolean = false;
    ngOnInit(): void {
        this.formGroup = new FormGroup(
            {
                email: this.fb.control(null, [Validators.required]),
                password: this.fb.control(null, [Validators.required, Validators.minLength(8)]),
                confirm_password: this.fb.control(''),
            },
            { validators: this.passwordMatchValidator() }
        );
    }

    passwordMatchValidator(): ValidatorFn {
        return (control: AbstractControl): ValidationErrors | null => {
            if (!this.signup) {
                return null;
            }
            const password = control.get('password')?.value;
            const confirmPassword = control.get('confirm_password')?.value;

            if (password !== confirmPassword) {
                control.get('confirm_password')?.setErrors({ missMatch: true });
                return { missMatch: true };
            }
            control.get('confirm_password')?.setErrors(null);
            return null;
        };
    }

    showLogin() {
        this.formGroup.get('confirm_password')?.clearValidators();
        this.formGroup.get('email')?.clearValidators();
        this.formGroup.get('email')?.addValidators([Validators.required]);
        this.formGroup.reset();
        this.login = true;
        this.signup = false;
    }

    showSignup() {
        this.formGroup
            .get('confirm_password')
            ?.addValidators([Validators.required, Validators.minLength(8), this.passwordValidators]);
        this.formGroup.get('email')?.addValidators([Validators.required, Validators.email]);
        this.formGroup.reset();
        this.login = false;
        this.signup = true;
    }

    passwordValidators(formArray: AbstractControl): ValidationErrors | null {
        const samePassword = formArray.get('password')?.value === formArray.get('confirm_password')?.value;
        return { samePassword: samePassword };
    }

    submit() {
        console.log('formGroup : ', this.formGroup.value);
        if (this.signup) {
            this.userService
                .createUser(this.formGroup.get('email')?.value, this.formGroup.get('password')?.value)
                .subscribe({
                    next: (data) => {
                        Swal.fire({
                            position: 'top-end',
                            icon: 'success',
                            title: 'Your account has been created',
                            showConfirmButton: false,
                            timer: 1000,
                        });
                        this.showLogin();
                    },
                });
        } else if (this.login) {
            this.userService
                .loginUser(this.formGroup.get('email')?.value, this.formGroup.get('password')?.value)
                .subscribe({
                    next: (data: any) => {
                        if (data['success']) {
                            this.router.navigate(['/profile']);
                        }
                    },
                });
        }
    }

    checkValidity(controlName: string) {
        const control = this.formGroup.get(controlName);
        if (control) {
            control.markAsTouched();
        }
    }

    matchPasswords(control: any): { [key: string]: boolean } | null {
        if (this.formGroup) {
            return control.value === this.formGroup.get('password')?.value ? null : { mismatch: true };
        }
        return null;
    }
}
